@@ -28,8 +28,6 @@ DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- * $Id$
  */
 
 /**
@@ -163,12 +161,11 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 	LONG RFUnBindFunctions(READER_CONTEXT *);
 	LONG RFUnloadReader(READER_CONTEXT *);
 	LONG RFInitializeReader(READER_CONTEXT *);
-	LONG RFUnInitializeReader(READER_CONTEXT *);
+	void RFUnInitializeReader(READER_CONTEXT *);
 	SCARDHANDLE RFCreateReaderHandle(READER_CONTEXT *);
-	LONG RFDestroyReaderHandle(SCARDHANDLE hCard);
 	LONG RFAddReaderHandle(READER_CONTEXT *, SCARDHANDLE);
 	LONG RFRemoveReaderHandle(READER_CONTEXT *, SCARDHANDLE);
-	LONG RFSetReaderEventState(READER_CONTEXT *, DWORD);
+	void RFSetReaderEventState(READER_CONTEXT *, DWORD);
 	LONG RFCheckReaderEventState(READER_CONTEXT *, SCARDHANDLE);
 	LONG RFClearReaderEventState(READER_CONTEXT *, SCARDHANDLE);
 	LONG RFCheckReaderStatus(READER_CONTEXT *);
