@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007,2008,2009 Mij <mij@bitchx.it>
+ * Copyright (c) 2007,2008,2009,2010,2011 Mij <mij@bitchx.it>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -19,30 +19,86 @@
  * SimCList library. See http://mij.oltrelinux.com/devel/simclist
  */
 
+/* SimCList implementation, version 1.6 */
+
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>      /* for setting errno */
-#include <inttypes.h>   /* (u)int*_t */
+#include <errno.h>          /* for setting errno */
 #include <sys/types.h>
-#include <sys/uio.h>    /* for READ_ERRCHECK() and write() */
-#include <fcntl.h>      /* for open() etc */
-#include <arpa/inet.h>  /* for htons() */
-#include <unistd.h>
-#include <time.h>       /* for time() for random seed */
-#include <sys/time.h>   /* for gettimeofday() */
-#include <sys/stat.h>   /* for open()'s access modes S_IRUSR etc */
+#ifndef _WIN32
+    /* not in Windows! */
+#   include <unistd.h>
+#   include <stdint.h>
+#endif
+#ifndef SIMCLIST_NO_DUMPRESTORE
+    /* includes for dump/restore */
+#   include <time.h>
+#   include <sys/uio.h>     /* for READ_ERRCHECK() and write() */
+#   include <fcntl.h>       /* for open() etc */
+#   ifndef _WIN32
+#       include <arpa/inet.h>  /* for htons() on UNIX */
+#   else
+#       include <winsock2.h>  /* for htons() on Windows */
+#   endif
+#endif
+
+/* disable asserts */
+#ifndef SIMCLIST_DEBUG
+#define NDEBUG
+#endif
+
+#include <assert.h>
+
+
+#include <sys/stat.h>       /* for open()'s access modes S_IRUSR etc */
 #include <limits.h>
-#include <stdint.h>
 
+#if defined(_MSC_VER) || defined(__MINGW32__)
+/* provide gettimeofday() missing in Windows */
+int gettimeofday(struct timeval *tp, void *tzp) {
+    DWORD t;
 
-/* use rand() in place of srand()? */
-#ifndef _BSD_SOURCE
-#   define random       rand
-#   define srandom      srand
+    /* XSI says: "If tzp is not a null pointer, the behavior is unspecified" */
+    assert(tzp == NULL);
+
+    t = timeGetTime();
+    tp->tv_sec = t / 1000;
+    tp->tv_usec = t % 1000;
+    return 0;
+}
 #endif
 
 
+/* work around lack of inttypes.h support in broken Microsoft Visual Studio compilers */
+#if !defined(_WIN32) || !defined(_MSC_VER)
+#   include <inttypes.h>   /* (u)int*_t */
+#else
+#   include <basetsd.h>
+typedef UINT8   uint8_t;
+typedef UINT16  uint16_t;
+typedef ULONG32 uint32_t;
+typedef UINT64  uint64_t;
+typedef INT8    int8_t;
+typedef INT16   int16_t;
+typedef LONG32  int32_t;
+typedef INT64   int64_t;
+#endif
+
+
+/* define some commodity macros for Dump/Restore functionality */
 #ifndef SIMCLIST_NO_DUMPRESTORE
+/* write() decorated with error checking logic */
+#define WRITE_ERRCHECK(fd, msgbuf, msglen)      do {                                                    \
+                                                    if (write(fd, msgbuf, msglen) < 0) return -1;       \
+                                                } while (0);
+/* READ_ERRCHECK() decorated with error checking logic */
+#define READ_ERRCHECK(fd, msgbuf, msglen)      do {                                                     \
+                                                    if (read(fd, msgbuf, msglen) != msglen) {           \
+                                                        /*errno = EPROTO;*/                             \
+                                                        return -1;                                      \
+                                                    }                                                   \
+                                                } while (0);
+
 /* convert 64bit integers from host to network format */
 #define hton64(x)       (\
         htons(1) == 1 ?                                         \
@@ -67,13 +123,6 @@
 #define EPROTO  EIO
 #endif
 
-/* disable asserts */
-#ifndef SIMCLIST_DEBUG
-#define NDEBUG
-#endif
-
-#include <assert.h>
-
 #ifdef SIMCLIST_WITH_THREADS
 /* limit (approx) to the number of threads running
  * for threaded operations. Only meant when
@@ -129,7 +178,8 @@
 /* header for a list dump */
 struct list_dump_header_s {
     uint16_t ver;               /* version */
-    int64_t timestamp;          /* dump timestamp */
+    int32_t timestamp_sec;      /* dump timestamp, seconds since UNIX Epoch */
+    int32_t timestamp_usec;     /* dump timestamp, microseconds since timestamp_sec */
     int32_t rndterm;            /* random value terminator -- terminates the data sequence */
 
     uint32_t totlistlen;        /* sum of every element' size, bytes */
@@ -155,11 +205,11 @@ static int list_attrOk(const list_t *restrict l);
 #endif
 
 /* do not inline, this is recursive */
-static void list_sort_quicksort(list_t *restrict l, int versus, 
+static void list_sort_quicksort(list_t *restrict l, int versus,
         unsigned int first, struct list_entry_s *fel,
         unsigned int last, struct list_entry_s *lel);
 
-static inline void list_sort_selectionsort(list_t *restrict l, int versus, 
+static inline void list_sort_selectionsort(list_t *restrict l, int versus,
         unsigned int first, struct list_entry_s *fel,
         unsigned int last, struct list_entry_s *lel);
 
@@ -167,30 +217,65 @@ static void *list_get_minmax(const list_t *restrict l, int versus);
 
 static inline struct list_entry_s *list_findpos(const list_t *restrict l, int posstart);
 
-/* write() decorated with error checking logic */
-#define WRITE_ERRCHECK(fd, msgbuf, msglen)      do {                                                    \
-                                                    if (write(fd, msgbuf, msglen) < 0) return -1;       \
-                                                } while (0);
-/* READ_ERRCHECK() decorated with error checking logic */
-#define READ_ERRCHECK(fd, msgbuf, msglen)      do {                                                     \
-                                                    if (read(fd, msgbuf, msglen) != msglen) {           \
-                                                        /*errno = EPROTO;*/                             \
-                                                        return -1;                                      \
-                                                    }                                                   \
-                                                } while (0);
+/*
+ * Random Number Generator
+ *
+ * The user is expected to seed the RNG (ie call srand()) if
+ * SIMCLIST_SYSTEM_RNG is defined.
+ *
+ * Otherwise, a self-contained RNG based on LCG is used; see
+ * http://en.wikipedia.org/wiki/Linear_congruential_generator .
+ *
+ * Facts pro local RNG:
+ * 1. no need for the user to call srand() on his own
+ * 2. very fast, possibly faster than OS
+ * 3. avoid interference with user's RNG
+ *
+ * Facts pro system RNG:
+ * 1. may be more accurate (irrelevant for SimCList randno purposes)
+ * 2. why reinvent the wheel
+ *
+ * Default to local RNG for user's ease of use.
+ */
+
+#ifdef SIMCLIST_SYSTEM_RNG
+/* keep track whether we initialized already (non-0) or not (0) */
+static unsigned random_seed = 0;
+
+/* use local RNG */
+static inline void seed_random(void) {
+    if (random_seed == 0)
+        random_seed = (unsigned)getpid() ^ (unsigned)time(NULL);
+}
+
+static inline long get_random(void) {
+    random_seed = (1664525 * random_seed + 1013904223);
+    return random_seed;
+}
+
+#else
+/* use OS's random generator */
+#   define  seed_random()
+#   define  get_random()        (rand())
+#endif
 
 
 /* list initialization */
 int list_init(list_t *restrict l) {
     if (l == NULL) return -1;
 
-    srandom((unsigned long)time(NULL));
+    memset(l, 0, sizeof *l);
+
+    seed_random();
 
     l->numels = 0;
 
     /* head/tail sentinels and mid pointer */
     l->head_sentinel = (struct list_entry_s *)malloc(sizeof(struct list_entry_s));
     l->tail_sentinel = (struct list_entry_s *)malloc(sizeof(struct list_entry_s));
+    if (NULL == l->tail_sentinel || NULL == l->head_sentinel)
+        return -1;
+
     l->head_sentinel->next = l->tail_sentinel;
     l->tail_sentinel->prev = l->head_sentinel;
     l->head_sentinel->prev = l->tail_sentinel->next = l->mid = NULL;
@@ -204,12 +289,15 @@ int list_init(list_t *restrict l) {
     /* free-list attributes */
     l->spareels = (struct list_entry_s **)malloc(SIMCLIST_MAX_SPARE_ELEMS * sizeof(struct list_entry_s *));
     l->spareelsnum = 0;
+    if (NULL == l->spareels)
+        return -1;
 
 #ifdef SIMCLIST_WITH_THREADS
     l->threadcount = 0;
 #endif
 
-    list_attributes_setdefaults(l);
+    if (list_attributes_setdefaults(l))
+        return -1;
 
     assert(list_repOk(l));
     assert(list_attrOk(l));
@@ -244,7 +332,7 @@ int list_attributes_setdefaults(list_t *restrict l) {
     l->attrs.unserializer = NULL;
 
     assert(list_attrOk(l));
-    
+
     return 0;
 }
 
@@ -255,7 +343,7 @@ int list_attributes_comparator(list_t *restrict l, element_comparator comparator
     l->attrs.comparator = comparator_fun;
 
     assert(list_attrOk(l));
-    
+
     return 0;
 }
 
@@ -339,7 +427,7 @@ static void *list_get_minmax(const list_t *restrict l, int versus) {
 
     if (l->attrs.comparator == NULL || l->numels == 0)
         return NULL;
-    
+
     curminmax = l->head_sentinel->next->data;
     for (s = l->head_sentinel->next->next; s != l->tail_sentinel; s = s->next) {
         if (l->attrs.comparator(curminmax, s->data) * versus > 0)
@@ -355,6 +443,9 @@ static inline struct list_entry_s *list_findpos(const list_t *restrict l, int po
     float x;
     int i;
 
+    if (NULL == l->head_sentinel || NULL == l->tail_sentinel)
+		return NULL;
+
     /* accept 1 slot overflow for fetching head and tail sentinels */
     if (posstart < -1 || posstart > (int)l->numels) return NULL;
 
@@ -379,16 +470,19 @@ static inline struct list_entry_s *list_findpos(const list_t *restrict l, int po
 void *list_extract_at(list_t *restrict l, unsigned int pos) {
     struct list_entry_s *tmp;
     void *data;
-    
+
     if (l->iter_active || pos >= l->numels) return NULL;
 
     tmp = list_findpos(l, pos);
+    if (NULL == tmp)
+        return NULL;
+
     data = tmp->data;
 
     tmp->data = NULL;   /* save data from list_drop_elem() free() */
     list_drop_elem(l, tmp, pos);
     l->numels--;
-    
+
     assert(list_repOk(l));
 
     return data;
@@ -396,9 +490,9 @@ void *list_extract_at(list_t *restrict l, unsigned int pos) {
 
 int list_insert_at(list_t *restrict l, const void *data, unsigned int pos) {
     struct list_entry_s *lent, *succ, *prec;
-    
+
     if (l->iter_active || pos > l->numels) return -1;
-    
+
     /* this code optimizes malloc() with a free-list */
     if (l->spareelsnum > 0) {
         lent = l->spareels[l->spareelsnum-1];
@@ -413,6 +507,11 @@ int list_insert_at(list_t *restrict l, const void *data, unsigned int pos) {
         /* make room for user' data (has to be copied) */
         size_t datalen = l->attrs.meter(data);
         lent->data = (struct list_entry_s *)malloc(datalen);
+        if (NULL == lent->data)
+        {
+            free(lent);
+            return -1;
+        }
         memcpy(lent->data, data, datalen);
     } else {
         lent->data = (void*)data;
@@ -420,8 +519,14 @@ int list_insert_at(list_t *restrict l, const void *data, unsigned int pos) {
 
     /* actually append element */
     prec = list_findpos(l, pos-1);
+    if (NULL == prec)
+    {
+        free(lent->data);
+        free(lent);
+        return -1;
+    }
     succ = prec->next;
-    
+
     prec->next = lent;
     lent->prev = prec;
     lent->next = succ;
@@ -448,11 +553,11 @@ int list_delete(list_t *restrict l, const void *data) {
 
 	pos = list_locate(l, data);
 	if (pos < 0)
-		return pos;
+		return -1;
 
 	r = list_delete_at(l, pos);
 	if (r < 0)
-		return r;
+		return -1;
 
     assert(list_repOk(l));
 
@@ -479,16 +584,17 @@ int list_delete_at(list_t *restrict l, unsigned int pos) {
 
 int list_delete_range(list_t *restrict l, unsigned int posstart, unsigned int posend) {
     struct list_entry_s *lastvalid, *tmp, *tmp2;
-    unsigned int i;
+    unsigned int numdel, midposafter, i;
     int movedx;
-    unsigned int numdel, midposafter;
 
     if (l->iter_active || posend < posstart || posend >= l->numels) return -1;
 
+    numdel = posend - posstart + 1;
+    if (numdel == l->numels) return list_clear(l);
+
     tmp = list_findpos(l, posstart);    /* first el to be deleted */
     lastvalid = tmp->prev;              /* last valid element */
 
-    numdel = posend - posstart + 1;
     midposafter = (l->numels-1-numdel)/2;
 
     midposafter = midposafter < posstart ? midposafter : midposafter+numdel;
@@ -536,49 +642,55 @@ int list_delete_range(list_t *restrict l, unsigned int posstart, unsigned int po
 
     assert(list_repOk(l));
 
-    return 0;
+    return numdel;
 }
 
 int list_clear(list_t *restrict l) {
     struct list_entry_s *s;
+    unsigned int numels;
+
+    /* will be returned */
+    numels = l->numels;
 
     if (l->iter_active) return -1;
-    
-    if (l->attrs.copy_data) {        /* also free user data */
-        /* spare a loop conditional with two loops: spareing elems and freeing elems */
-        for (s = l->head_sentinel->next; l->spareelsnum < SIMCLIST_MAX_SPARE_ELEMS && s != l->tail_sentinel; s = s->next) {
-            /* move elements as spares as long as there is room */
-            if (s->data != NULL) free(s->data);
-            l->spareels[l->spareelsnum++] = s;
+
+    if (l->head_sentinel && l->tail_sentinel) {
+        if (l->attrs.copy_data) {        /* also free user data */
+            /* spare a loop conditional with two loops: spareing elems and freeing elems */
+            for (s = l->head_sentinel->next; l->spareelsnum < SIMCLIST_MAX_SPARE_ELEMS && s != l->tail_sentinel; s = s->next) {
+                /* move elements as spares as long as there is room */
+                if (s->data != NULL) free(s->data);
+                l->spareels[l->spareelsnum++] = s;
+            }
+            while (s != l->tail_sentinel) {
+                /* free the remaining elems */
+                if (s->data != NULL) free(s->data);
+                s = s->next;
+                free(s->prev);
+            }
+            l->head_sentinel->next = l->tail_sentinel;
+            l->tail_sentinel->prev = l->head_sentinel;
+        } else { /* only free element containers */
+            /* spare a loop conditional with two loops: spareing elems and freeing elems */
+            for (s = l->head_sentinel->next; l->spareelsnum < SIMCLIST_MAX_SPARE_ELEMS && s != l->tail_sentinel; s = s->next) {
+                /* move elements as spares as long as there is room */
+                l->spareels[l->spareelsnum++] = s;
+            }
+            while (s != l->tail_sentinel) {
+                /* free the remaining elems */
+                s = s->next;
+                free(s->prev);
+            }
+            l->head_sentinel->next = l->tail_sentinel;
+            l->tail_sentinel->prev = l->head_sentinel;
         }
-        while (s != l->tail_sentinel) {
-            /* free the remaining elems */
-            if (s->data != NULL) free(s->data);
-            s = s->next;
-            free(s->prev);
-        }
-        l->head_sentinel->next = l->tail_sentinel;
-        l->tail_sentinel->prev = l->head_sentinel;
-    } else { /* only free element containers */
-        /* spare a loop conditional with two loops: spareing elems and freeing elems */
-        for (s = l->head_sentinel->next; l->spareelsnum < SIMCLIST_MAX_SPARE_ELEMS && s != l->tail_sentinel; s = s->next) {
-            /* move elements as spares as long as there is room */
-            l->spareels[l->spareelsnum++] = s;
-        }
-        while (s != l->tail_sentinel) {
-            /* free the remaining elems */
-            s = s->next;
-            free(s->prev);
-        }
-        l->head_sentinel->next = l->tail_sentinel;
-        l->tail_sentinel->prev = l->head_sentinel;
     }
     l->numels = 0;
     l->mid = NULL;
 
     assert(list_repOk(l));
 
-    return 0;
+    return numels;
 }
 
 unsigned int list_size(const list_t *restrict l) {
@@ -592,7 +704,10 @@ int list_empty(const list_t *restrict l) {
 int list_locate(const list_t *restrict l, const void *data) {
     struct list_entry_s *el;
     int pos = 0;
-    
+
+    if (NULL == l->head_sentinel || NULL == l->tail_sentinel)
+		return -1;
+
     if (l->attrs.comparator != NULL) {
         /* use comparator */
         for (el = l->head_sentinel->next; el != l->tail_sentinel; el = el->next, pos++) {
@@ -614,6 +729,9 @@ void *list_seek(list_t *restrict l, const void *indicator) {
 
     if (l->attrs.seeker == NULL) return NULL;
 
+    if (NULL == l->head_sentinel || NULL == l->tail_sentinel)
+		return NULL;
+
     for (iter = l->head_sentinel->next; iter != l->tail_sentinel; iter = iter->next) {
         if (l->attrs.seeker(iter->data, indicator) != 0) return iter->data;
     }
@@ -634,7 +752,12 @@ int list_concat(const list_t *l1, const list_t *l2, list_t *restrict dest) {
     if (l1 == NULL || l2 == NULL || dest == NULL || l1 == dest || l2 == dest)
         return -1;
 
-    list_init(dest);
+    if (NULL == l1->head_sentinel || NULL == l1->tail_sentinel
+		|| NULL == l2->head_sentinel || NULL == l2->tail_sentinel)
+		return -1;
+
+    if (list_init(dest))
+        return -1;
 
     dest->numels = l1->numels + l2->numels;
     if (dest->numels == 0)
@@ -645,6 +768,8 @@ int list_concat(const list_t *l1, const list_t *l2, list_t *restrict dest) {
     el = dest->head_sentinel;
     while (srcel != l1->tail_sentinel) {
         el->next = (struct list_entry_s *)malloc(sizeof(struct list_entry_s));
+        if (NULL == el->next)
+            return -1;
         el->next->prev = el;
         el = el->next;
         el->data = srcel->data;
@@ -655,6 +780,8 @@ int list_concat(const list_t *l1, const list_t *l2, list_t *restrict dest) {
     srcel = l2->head_sentinel->next;
     while (srcel != l2->tail_sentinel) {
         el->next = (struct list_entry_s *)malloc(sizeof(struct list_entry_s));
+        if (NULL == el->next)
+            return -1;
         el->next->prev = el;
         el = el->next;
         el->data = srcel->data;
@@ -662,7 +789,7 @@ int list_concat(const list_t *l1, const list_t *l2, list_t *restrict dest) {
     }
     el->next = dest->tail_sentinel;
     dest->tail_sentinel->prev = el;
-    
+
     /* fix mid pointer */
     err = l2->numels - l1->numels;
     if ((err+1)/2 > 0) {        /* correct pos RIGHT (err-1)/2 moves */
@@ -672,7 +799,7 @@ int list_concat(const list_t *l1, const list_t *l2, list_t *restrict dest) {
         err = -err/2;
         for (cnt = 0; cnt < (unsigned int)err; cnt++) dest->mid = dest->mid->prev;
     }
- 
+
     assert(!(list_repOk(l1) && list_repOk(l2)) || list_repOk(dest));
 
     return 0;
@@ -684,6 +811,10 @@ int list_sort(list_t *restrict l, int versus) {
 
     if (l->numels <= 1)
         return 0;
+
+    if (NULL == l->head_sentinel || NULL == l->tail_sentinel)
+		return -1;
+
     list_sort_quicksort(l, versus, 0, l->head_sentinel->next, l->numels-1, l->tail_sentinel->prev);
     assert(list_repOk(l));
     return 0;
@@ -706,7 +837,7 @@ static void *list_sort_quicksort_threadwrapper(void *wrapped_params) {
 }
 #endif
 
-static inline void list_sort_selectionsort(list_t *restrict l, int versus, 
+static inline void list_sort_selectionsort(list_t *restrict l, int versus,
         unsigned int first, struct list_entry_s *fel,
         unsigned int last, struct list_entry_s *lel) {
     struct list_entry_s *cursor, *toswap, *firstunsorted;
@@ -714,7 +845,7 @@ static inline void list_sort_selectionsort(list_t *restrict l, int versus,
 
     if (last <= first) /* <= 1-element lists are always sorted */
         return;
-    
+
     for (firstunsorted = fel; firstunsorted != lel; firstunsorted = firstunsorted->next) {
         /* find min or max in the remainder of the list */
         for (toswap = firstunsorted, cursor = firstunsorted->next; cursor != lel->next; cursor = cursor->next)
@@ -727,7 +858,7 @@ static inline void list_sort_selectionsort(list_t *restrict l, int versus,
     }
 }
 
-static void list_sort_quicksort(list_t *restrict l, int versus, 
+static void list_sort_quicksort(list_t *restrict l, int versus,
         unsigned int first, struct list_entry_s *fel,
         unsigned int last, struct list_entry_s *lel) {
     unsigned int pivotid;
@@ -743,16 +874,16 @@ static void list_sort_quicksort(list_t *restrict l, int versus,
 
     if (last <= first)      /* <= 1-element lists are always sorted */
         return;
-    
+
     if (last - first+1 <= SIMCLIST_MINQUICKSORTELS) {
         list_sort_selectionsort(l, versus, first, fel, last, lel);
         return;
     }
-    
+
     /* base of iteration: one element list */
     if (! (last > first)) return;
 
-    pivotid = (random() % (last - first + 1));
+    pivotid = (get_random() % (last - first + 1));
     /* pivotid = (last - first + 1) / 2; */
 
     /* find pivot */
@@ -822,6 +953,8 @@ static void list_sort_quicksort(list_t *restrict l, int versus,
         /* prepare wrapped args, then start thread */
         if (l->threadcount < SIMCLIST_MAXTHREADS-1) {
             struct list_sort_wrappedparams *wp = (struct list_sort_wrappedparams *)malloc(sizeof(struct list_sort_wrappedparams));
+            if (NULL == wp)
+                return -1;
             l->threadcount++;
             traised = 1;
             wp->l = l;
@@ -852,6 +985,8 @@ static void list_sort_quicksort(list_t *restrict l, int versus,
 
 int list_iterator_start(list_t *restrict l) {
     if (l->iter_active) return 0;
+    if (NULL == l->head_sentinel)
+		return -1;
     l->iter_pos = 0;
     l->iter_active = 1;
     l->iter_curentry = l->head_sentinel->next;
@@ -885,7 +1020,7 @@ int list_iterator_stop(list_t *restrict l) {
 int list_hash(const list_t *restrict l, list_hash_t *restrict hash) {
     struct list_entry_s *x;
     list_hash_t tmphash;
-    
+
     assert(hash != NULL);
 
     tmphash = l->numels * 2 + 100;
@@ -909,7 +1044,7 @@ int list_hash(const list_t *restrict l, list_hash_t *restrict hash) {
         /* hash each element with the user-given function */
         for (x = l->head_sentinel->next; x != l->tail_sentinel; x = x->next) {
             tmphash += tmphash ^ l->attrs.hasher(x->data);
-            tmphash +=* hash % l->numels;
+            tmphash += tmphash % l->numels;
         }
     }
 
@@ -933,9 +1068,11 @@ int list_dump_getinfo_filedescriptor(int fd, list_dump_info_t *restrict info) {
         return -1;
     }
 
-    /* timestamp */
-    READ_ERRCHECK(fd, & info->timestamp, sizeof(info->timestamp));
-    info->timestamp = hton64(info->timestamp);
+    /* timestamp.tv_sec and timestamp.tv_usec */
+    READ_ERRCHECK(fd, & info->timestamp.tv_sec, sizeof(info->timestamp.tv_sec));
+    info->timestamp.tv_sec = ntohl(info->timestamp.tv_sec);
+    READ_ERRCHECK(fd, & info->timestamp.tv_usec, sizeof(info->timestamp.tv_usec));
+    info->timestamp.tv_usec = ntohl(info->timestamp.tv_usec);
 
     /* list terminator (to check thereafter) */
     READ_ERRCHECK(fd, & terminator_head, sizeof(terminator_head));
@@ -1008,13 +1145,13 @@ int list_dump_filedescriptor(const list_t *restrict l, int fd, size_t *restrict
     /****       DUMP FORMAT      ****
 
     [ ver   timestamp   |  totlen   numels  elemlen     hash    |   DATA ]
-    
+
     where DATA can be:
     @ for constant-size list (element size is constant; elemlen > 0)
     [ elem    elem    ...    elem ]
     @ for other lists (element size dictated by element_meter each time; elemlen <= 0)
     [ size elem     size elem       ...     size elem ]
-    
+
     all integers are encoded in NETWORK BYTE FORMAT
     *****/
 
@@ -1025,10 +1162,10 @@ int list_dump_filedescriptor(const list_t *restrict l, int fd, size_t *restrict
 
     /* timestamp */
     gettimeofday(&timeofday, NULL);
-    header.timestamp = (int64_t)timeofday.tv_sec * 1000000 + (int64_t)timeofday.tv_usec;
-    header.timestamp = hton64(header.timestamp);
+    header.timestamp_sec = htonl(timeofday.tv_sec);
+    header.timestamp_usec = htonl(timeofday.tv_usec);
 
-    header.rndterm = htonl((int32_t)random());
+    header.rndterm = htonl((int32_t)get_random());
 
     /* total list size is postprocessed afterwards */
 
@@ -1056,7 +1193,7 @@ int list_dump_filedescriptor(const list_t *restrict l, int fd, size_t *restrict
     /* write CONTENT */
     if (l->numels > 0) {
         /* SPECULATE that the list has constant element size */
-        
+
         if (l->attrs.serializer != NULL) {  /* user user-specified serializer */
             /* get preliminary length of serialized element in header.elemlen */
             ser_buf = l->attrs.serializer(l->head_sentinel->next->data, & header.elemlen);
@@ -1072,6 +1209,10 @@ int list_dump_filedescriptor(const list_t *restrict l, int fd, size_t *restrict
                         header.elemlen = 0;
                         header.totlistlen = 0;
                         x = l->head_sentinel;
+                        if (lseek(fd, SIMCLIST_DUMPFORMAT_HEADERLEN, SEEK_SET) < 0) {
+                            /* errno set by lseek() */
+                            return -1;
+                        }
                         /* restart from the beginning */
                         continue;
                     }
@@ -1119,13 +1260,14 @@ int list_dump_filedescriptor(const list_t *restrict l, int fd, size_t *restrict
     lseek(fd, 0, SEEK_SET);
 
     WRITE_ERRCHECK(fd, & header.ver, sizeof(header.ver));                        /* version */
-    WRITE_ERRCHECK(fd, & header.timestamp, sizeof(header.timestamp));            /* timestamp */
+    WRITE_ERRCHECK(fd, & header.timestamp_sec, sizeof(header.timestamp_sec));    /* timestamp seconds */
+    WRITE_ERRCHECK(fd, & header.timestamp_usec, sizeof(header.timestamp_usec));  /* timestamp microseconds */
     WRITE_ERRCHECK(fd, & header.rndterm, sizeof(header.rndterm));                /* random terminator */
 
     WRITE_ERRCHECK(fd, & header.totlistlen, sizeof(header.totlistlen));          /* total length of elements */
     WRITE_ERRCHECK(fd, & header.numels, sizeof(header.numels));                  /* number of elements */
     WRITE_ERRCHECK(fd, & header.elemlen, sizeof(header.elemlen));                /* size of each element, or 0 for independent */
-    WRITE_ERRCHECK(fd, & header.listhash, sizeof(header.listhash));             /* list hash, or 0 for "ignore" */
+    WRITE_ERRCHECK(fd, & header.listhash, sizeof(header.listhash));              /* list hash, or 0 for "ignore" */
 
 
     /* possibly store total written length in "len" */
@@ -1145,7 +1287,7 @@ int list_restore_filedescriptor(list_t *restrict l, int fd, size_t *restrict len
     memset(& header, 0, sizeof(header));
 
     /* read header */
-    
+
     /* version */
     READ_ERRCHECK(fd, &header.ver, sizeof(header.ver));
     header.ver = ntohs(header.ver);
@@ -1155,7 +1297,10 @@ int list_restore_filedescriptor(list_t *restrict l, int fd, size_t *restrict len
     }
 
     /* timestamp */
-    READ_ERRCHECK(fd, & header.timestamp, sizeof(header.timestamp));
+    READ_ERRCHECK(fd, & header.timestamp_sec, sizeof(header.timestamp_sec));
+    header.timestamp_sec = ntohl(header.timestamp_sec);
+    READ_ERRCHECK(fd, & header.timestamp_usec, sizeof(header.timestamp_usec));
+    header.timestamp_usec = ntohl(header.timestamp_usec);
 
     /* list terminator */
     READ_ERRCHECK(fd, & header.rndterm, sizeof(header.rndterm));
@@ -1181,11 +1326,13 @@ int list_restore_filedescriptor(list_t *restrict l, int fd, size_t *restrict len
 
     /* read content */
     totreadlen = totmemorylen = 0;
-    if (header.elemlen > 0) {   
+    if (header.elemlen > 0) {
         /* elements have constant size = header.elemlen */
         if (l->attrs.unserializer != NULL) {
             /* use unserializer */
             buf = malloc(header.elemlen);
+            if (NULL == buf)
+                return -1;
             for (cnt = 0; cnt < header.numels; cnt++) {
                 READ_ERRCHECK(fd, buf, header.elemlen);
                 list_append(l, l->attrs.unserializer(buf, & elsize));
@@ -1195,6 +1342,8 @@ int list_restore_filedescriptor(list_t *restrict l, int fd, size_t *restrict len
             /* copy verbatim into memory */
             for (cnt = 0; cnt < header.numels; cnt++) {
                 buf = malloc(header.elemlen);
+                if (NULL == buf)
+                    return -1;
                 READ_ERRCHECK(fd, buf, header.elemlen);
                 list_append(l, buf);
             }
@@ -1208,6 +1357,8 @@ int list_restore_filedescriptor(list_t *restrict l, int fd, size_t *restrict len
             for (cnt = 0; cnt < header.numels; cnt++) {
                 READ_ERRCHECK(fd, & elsize, sizeof(elsize));
                 buf = malloc((size_t)elsize);
+                if (NULL == buf)
+                    return -1;
                 READ_ERRCHECK(fd, buf, elsize);
                 totreadlen += elsize;
                 list_append(l, l->attrs.unserializer(buf, & elsize));
@@ -1218,6 +1369,8 @@ int list_restore_filedescriptor(list_t *restrict l, int fd, size_t *restrict len
             for (cnt = 0; cnt < header.numels; cnt++) {
                 READ_ERRCHECK(fd, & elsize, sizeof(elsize));
                 buf = malloc(elsize);
+                if (NULL == buf)
+                    return -1;
                 READ_ERRCHECK(fd, buf, elsize);
                 totreadlen += elsize;
                 list_append(l, buf);
@@ -1258,29 +1411,34 @@ int list_restore_filedescriptor(list_t *restrict l, int fd, size_t *restrict len
 }
 
 int list_dump_file(const list_t *restrict l, const char *restrict filename, size_t *restrict len) {
-    int fd;
-    size_t sizetoret;
+    int fd, oflag, mode;
 
-    fd = open(filename, O_RDWR | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+#ifndef _WIN32
+    oflag = O_RDWR | O_CREAT | O_TRUNC;
+    mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
+#else
+    oflag = _O_RDWR | _O_CREAT | _O_TRUNC;
+    mode = _S_IRUSR | _S_IWUSR | _S_IRGRP | _S_IROTH;
+#endif
+    fd = open(filename, oflag, mode);
     if (fd < 0) return -1;
 
-    sizetoret = list_dump_filedescriptor(l, fd, len);
+    list_dump_filedescriptor(l, fd, len);
     close(fd);
 
-    return sizetoret;
+    return 0;
 }
 
 int list_restore_file(list_t *restrict l, const char *restrict filename, size_t *restrict len) {
     int fd;
-    size_t totdata;
 
     fd = open(filename, O_RDONLY, 0);
     if (fd < 0) return -1;
 
-    totdata = list_restore_filedescriptor(l, fd, len);
+    list_restore_filedescriptor(l, fd, len);
     close(fd);
 
-    return totdata;
+    return 0;
 }
 #endif /* ifndef SIMCLIST_NO_DUMPRESTORE */
 
@@ -1288,13 +1446,15 @@ int list_restore_file(list_t *restrict l, const char *restrict filename, size_t
 static int list_drop_elem(list_t *restrict l, struct list_entry_s *tmp, unsigned int pos) {
     if (tmp == NULL) return -1;
 
-    /* fix mid pointer */
+    /* fix mid pointer. This is wrt the PRE situation */
     if (l->numels % 2) {    /* now odd */
-        if (pos >= l->numels/2) l->mid = l->mid->prev;
+        /* sort out the base case by hand */
+        if (l->numels == 1) l->mid = NULL;
+        else if (pos >= l->numels/2) l->mid = l->mid->prev;
     } else {                /* now even */
         if (pos < l->numels/2) l->mid = l->mid->next;
     }
-    
+
     tmp->prev->next = tmp->next;
     tmp->next->prev = tmp->prev;
 
@@ -1302,7 +1462,7 @@ static int list_drop_elem(list_t *restrict l, struct list_entry_s *tmp, unsigned
     if (l->attrs.copy_data && tmp->data != NULL)
         free(tmp->data);
 
-    if (l->spareelsnum < SIMCLIST_MAX_SPARE_ELEMS) {
+    if (l->spareels != NULL && l->spareelsnum < SIMCLIST_MAX_SPARE_ELEMS) {
         l->spareels[l->spareelsnum++] = tmp;
     } else {
         free(tmp);
@@ -1312,7 +1472,7 @@ static int list_drop_elem(list_t *restrict l, struct list_entry_s *tmp, unsigned
 }
 
 /* ready-made comparators and meters */
-#define SIMCLIST_NUMBER_COMPARATOR(type)     int list_comparator_##type(const void *a, const void *b) { return( *(type *)a < *(type *)b) - (*(type *)a > *(type *)b); } 
+#define SIMCLIST_NUMBER_COMPARATOR(type)     int list_comparator_##type(const void *a, const void *b) { return( *(type *)a < *(type *)b) - (*(type *)a > *(type *)b); }
 
 SIMCLIST_NUMBER_COMPARATOR(int8_t)
 SIMCLIST_NUMBER_COMPARATOR(int16_t)
@@ -1330,7 +1490,7 @@ SIMCLIST_NUMBER_COMPARATOR(double)
 int list_comparator_string(const void *a, const void *b) { return strcmp((const char *)b, (const char *)a); }
 
 /* ready-made metric functions */
-#define SIMCLIST_METER(type)        size_t list_meter_##type(const void *el) { (void)el; return sizeof(type); }
+#define SIMCLIST_METER(type)        size_t list_meter_##type(const void *el) { if (el) { /* kill compiler whinge */ } return sizeof(type); }
 
 SIMCLIST_METER(int8_t)
 SIMCLIST_METER(int16_t)
@@ -1393,7 +1553,7 @@ static int list_repOk(const list_t *restrict l) {
             /* spare elements checks */
             l->spareelsnum <= SIMCLIST_MAX_SPARE_ELEMS
          );
-    
+
     if (!ok) return 0;
 
     if (l->numels >= 1) {
