@@ -45,10 +45,8 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 	int SYS_USleep(int);
 
-	int SYS_Daemon(int, int);
-
 	int SYS_RandomInt(int, int);
 
-	int SYS_GetSeed(void);
+	void SYS_InitRandom(void);
 
 #endif							/* __sys_generic_h__ */
